/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_MESHBUILDERSHAPEVISITOR_H
#define X3DTK_X3D_MESHBUILDERSHAPEVISITOR_H

#include "X3D_ShapeVisitor.h"
#include "X3D_MeshBuilderStateVariables.h"
#include "X3D_Appearance.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_FillProperties.h"
#include "X3D_LineProperties.h"
#include "X3D_Material.h"
#include "X3D_Shape.h"
#include "X3D_X3DGeometryNode.h"
#include "MESH_X3DGroupingNode.h"
#include "MESH_Shape.h"

namespace X3DTK {
namespace X3D {

class Shape;
class X3DAppearanceNode;

/*! \brief Class declared in <b><X3DTK/X3D/meshbuilder.h></b> providing a visitor
 *  of the Shape component for the X3D::TemplateMeshBuilder processor.
 *
 *  \ingroup X3D
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateMeshBuilderShapeVisitor : public ShapeVisitor
{
public:
  /// Constructor.
  TemplateMeshBuilderShapeVisitor();

  /// Enters a Shape.
  static void enterShape(Shape *S);
  /// Enters an X3DAppearanceNode.
  static void enterX3DAppearanceNode(X3DAppearanceNode *N);
  /// Walks on an X3DAppearanceNode.
  static bool walkOnX3DAppearanceNode(X3DAppearanceNode *N, SFNode child);
  /// Leaves a Shape.
  static void leaveShape(Shape *S);
  /// Leaves an X3DAppearanceNode.
  static void leaveX3DAppearanceNode(X3DAppearanceNode *N);
};

}
}

#include "X3D_MeshBuilderShapeVisitor.inl"

#endif
