/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_TRANSFORM_H
#define X3DTK_X3D_TRANSFORM_H

#include "X3DTypes.h"
#include "X3D_X3DGroupingNode.h"
#include "X3D_X3DBoundedObject.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/group.html">
 *  Grouping component</a> defining a Transform
 *  (<a href="../../doc/X3DSpecs/Part01/components/group.html#Transform">
 * documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class Transform : public X3DGroupingNode, public X3DBoundedObject
{
public:  
  /// Constructor.
  Transform();
  /// Destructor.
  virtual ~Transform();
  
  /// Sets the center of the transformation.
  void setCenter(const SFVec3f &center);
  /// Sets the rotation of the transformation.
  void setRotation(const SFRotation &rotation);
  /// Sets the scale of the transformation.
  void setScale(const SFVec3f &scale);
  /// Sets the scale orientation of the transformation.
  void setScaleOrientation(const SFRotation &scaleOrientation);
  /// Sets the translation of the transformation.
  void setTranslation(const SFVec3f &translation);

  /// Gets the center of the transformation.
  inline const SFVec3f &getCenter() const {return _center;};
  /// Gets the rotation of the transformation.
  inline const SFRotation &getRotation() const {return _rotation;};
  /// Gets the scale of the transformation.
  inline const SFVec3f &getScale() const {return _scale;};
  /// Gets the scale orientation of the transformation.
  inline const SFRotation &getScaleOrientation() const {return _scaleOrientation;};
  /// Gets the translation of the transformation.
  inline const SFVec3f &getTranslation() const {return _translation;};

private:
  /// Center attribute.
  SFVec3f _center;
  /// Rotation attribute.
  SFRotation _rotation;
  /// Scale attribute.
  SFVec3f _scale;
  /// Scale orientation attribute.
  SFRotation _scaleOrientation;
  /// Translation attribute.
  SFVec3f _translation;
};

}
}

#endif
