namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
MyX3DBuilderCoreVisitor<MData, VData, EData, FData, RW>::MyX3DBuilderCoreVisitor()
{
  define(Recorder<TemplateVertex<MData, VData, EData, FData, RW> >::getEnterFunction(&MyX3DBuilderCoreVisitor::enterVertex));
}

template<class MData, class VData, class EData, class FData, bool RW>
void MyX3DBuilderCoreVisitor<MData, VData, EData, FData, RW>::enterVertex(TemplateVertex<MData, VData, EData, FData, RW> *V)
{
  // StateVariables assignation.
  TemplateX3DBuilderStateVariables<MData, VData, EData, FData, RW> *stateVariables = Singleton<TemplateX3DBuilderStateVariables<MData, VData, EData, FData, RW> >::getInstance();

  // Test of the presence of MESH::VertexWeightData in VData.
  #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
  if (!VData::template find<VertexWeightData>())
  {
    // Call to super class function.
    TemplateX3DBuilderCoreVisitor<MData, VData, EData, FData, RW>::enterVertex(V);
  }
  else
  #endif
  {
    // Recoding the creation of the node.
    X3D::MyCoordinate *NC = dynamic_cast<X3D::MyCoordinate *>(stateVariables->getNode(V));
    if (NC == 0)
    {
      // Create a new MyCoordinate.
      NC = new X3D::MyCoordinate();
    
      const typename TemplateVertex<MData, VData, EData, FData, RW>::MFVertex &vertices = V->getVertices();
      
      // Test the optional presence of MESH::VertexPointData in VData.
      #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
      if (VData::template find<MESH::VertexPointData>())
      #endif
      {
        // Updating Coordinate.
        MFVec3f point;
        // Accessing the MESH::VertexPointData of VData.
        for (typename TemplateVertex<MData, VData, EData, FData, RW>::MFVertex::const_iterator v = vertices.begin(); v != vertices.end(); ++v)
          #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
          point.push_back(SFVec3f((*v)->template ogetData<MESH::VertexPointData>().getPoint()));
          #else
          point.push_back(SFVec3f((*v)->data().getPoint()));
          #endif
        NC->setPoint(point);
      }
      
      MFFloat weight;

      // Accessing the MESH::VertexWeightData of VData.
      #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
      if (VData::template find<MESH::VertexWeightData>())
      #endif
      {
        for (typename TemplateVertex<MData, VData, EData, FData, RW>::MFVertex::const_iterator v = vertices.begin(); v != vertices.end(); ++v)
        {
          #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
          weight.push_back((*v)->template ogetData<MESH::VertexWeightData>().getWeight());
          #else
          weight.push_back((*v)->data().getWeight());
          #endif
        }  
      }

      NC->setWeight(weight);

      // memorizing the couple (V, NC) to avoid to create a new Coordinate 
      // if V is visited a second time.
      stateVariables->addCoupleNode(V, NC);
    }
    stateVariables->pushNode(NC);
  }  
}

}
}
