namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW>
TransformComputerCoreVisitor<MData, VData, EData, FData, RW>::TransformComputerCoreVisitor()
{
  // Enter functions.
  define(Recorder<TemplateMesh<MData, VData, EData, FData, RW> >::getEnterFunction(&TransformComputerCoreVisitor::enterMesh));
  define(Recorder<Transform>::getEnterFunction(&TransformComputerCoreVisitor::enterTransform));
  // Leave function
  define(Recorder<X3DGroupingNode>::getLeaveFunction(&TransformComputerCoreVisitor::leaveX3DGroupingNode));
}

template<class MData, class VData, class EData, class FData, bool RW>
void TransformComputerCoreVisitor<MData, VData, EData, FData, RW>::enterMesh(TemplateMesh<MData, VData, EData, FData, RW> *M)
{
  // StateVariables assignation.
  TransformComputerStateVariables<MData, VData, EData, FData, RW> *stateVariables = Singleton<TransformComputerStateVariables<MData, VData, EData, FData, RW> >::getInstance();
  
  std::cout << "enter Mesh" << std::endl;
  std::cout << "Transform:" << std::endl << stateVariables->getMatrix() << std::endl;
  std::cout << "  number of vertices = " << M->getVertices().size() << std::endl;
  
  // Test the optional presence of MESH::VertexPointData in VData.
  #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
  if (VData::template find<MESH::VertexPointData>())
  #endif
  {
    // Accessing the MESH::VertexPointData of VData.
    for (typename TemplateMesh<MData, VData, EData, FData, RW>::MFVertex::const_iterator it = M->getVertices().begin(); it != M->getVertices().end(); ++it)
      #ifdef TEMPLATE_SPECIALIZATION_SUPPORTED
      std::cout << (*it)->template ogetData<MESH::VertexPointData>().getPoint() << std::endl;
      #else
      std::cout << (*it)->data().getPoint() << std::endl;
      #endif
  }    
}

template<class MData, class VData, class EData, class FData, bool RW>
void TransformComputerCoreVisitor<MData, VData, EData, FData, RW>::enterTransform(Transform *T)
{
  // Pushing the current transformation matrix on the stack.
  Singleton<TransformComputerStateVariables<MData, VData, EData, FData, RW> >::getInstance()->pushMatrix(T->getTransform());
}

template<class MData, class VData, class EData, class FData, bool RW>
void TransformComputerCoreVisitor<MData, VData, EData, FData, RW>::leaveX3DGroupingNode(X3DGroupingNode *)
{
  // Poping the current transformation matrix of the stack.
  Singleton<TransformComputerStateVariables<MData, VData, EData, FData, RW> >::getInstance()->popMatrix();
}

}
}
