#ifndef MYMESHDATA_H
#define MYMESHDATA_H

#include <X3DTK/MESH/scenegraph.h>
#include <X3DTK/simplifiedmeshscene.h>

namespace X3DTK {
namespace MESH {

// My vertex data structure containing the weight information.

class VertexDistanceData
{
public:
  void setDistance(float distance);
  inline float getDistance() const {return _distance;};
  
private:
  float _distance;  
};

// My Mesh data aggregates using recursive typelists.
// Typedefs are useful to simplify template syntax.
#ifdef TEMPLATE_SPECIALIZATION_SUPPORTED

typedef clist<tlist<VertexPointData, tlist<VertexNormalData, tlist<VertexColorData, tlist<VertexDistanceData> > > > > MyVertexData;
typedef clist<tlist<FaceNormalData, tlist<FaceColorData> > > MyFaceData;
typedef clist<tlist<MeshNormalData, tlist<MeshColorData> > > MyMeshData;

#else

class MyVertexData : public VertexData, public VertexDistanceData {};
class MyFaceData : public FaceData {};
class MyMeshData : public MeshData {};

#endif

// My MESH entities.

typedef TemplateMesh<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> MyMesh;
typedef TemplateVertex<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> MyVertex;
typedef SFTemplateVertex<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> MySFVertex;
typedef SFTemplateEdge<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> MySFEdge;
typedef SFTemplateFace<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> MySFFace;

// function computing the norm attribute of the vertices
void computeVerticesDistance(MyMesh *M);

}

// My SimpleMeshScene

typedef TemplateSimplifiedMeshScene<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> MySimplifiedMeshScene;

}

#endif
