#include "MESH_MyStructureComputerCoreVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

MyStructureComputerCoreVisitor::MyStructureComputerCoreVisitor()
{
  // Enter functions.
  define(Recorder<Mesh>::getEnterFunction(&MyStructureComputerCoreVisitor::enterMesh));
  define(Recorder<Transform>::getEnterFunction(&MyStructureComputerCoreVisitor::enterTransform));
  // Leave function.
  define(Recorder<X3DGroupingNode>::getLeaveFunction(&MyStructureComputerCoreVisitor::leaveX3DGroupingNode));
}

void MyStructureComputerCoreVisitor::enterMesh(Mesh *M)
{
  // StateVariables assignation.
  MyStructureComputerStateVariables *stateVariables = Singleton<MyStructureComputerStateVariables>::getInstance();
    
  SFMatrix34f T = stateVariables->getMatrix();

  // Beginning a new Mesh by memorizing the current vertex index.
  stateVariables->beginNewMesh();

  // filling the vertices.
  for (Mesh::MFVertex::const_iterator v = M->getVertices().begin(); v != M->getVertices().end(); ++v)
  {
    SFPoint3f P = T*(*v)->data().getPoint();
    stateVariables->addVertex(P.x, P.y, P.z);
  }
  
  // filling the faces.
  for (Mesh::MFFace::const_iterator f = M->getFaces().begin(); f != M->getFaces().end(); ++f)
  {
    const SFFace::MFEdge &edges = (*f)->getEdges();
    list<unsigned int> indexList;
    for (SFFace::MFEdge::const_iterator e = edges.begin(); e != edges.end(); ++e)
      indexList.push_back((*e)->getFromVertex()->getIndex());
    
    stateVariables->addFace(indexList);
  }  
}

void MyStructureComputerCoreVisitor::enterTransform(Transform *T)
{
  Singleton<MyStructureComputerStateVariables>::getInstance()->pushMatrix(T->getTransform());
}

void MyStructureComputerCoreVisitor::leaveX3DGroupingNode(X3DGroupingNode *)
{
  Singleton<MyStructureComputerStateVariables>::getInstance()->popMatrix();
}

}
}
